# -*- coding: utf-8 -*-
import ftfy
import hao
import opencc
import regex

CC = opencc.OpenCC('t2s')

P_DATES = hao.regexes.re_compile([
    r'(?:一九|二零|二〇|二O|二0)[O0零〇一二三四五六七八九]{2}\s*年\s*十?[一二三四五六七八九十]\s*月\s*(?:(?:十|二十|三十)?[一二三四五六七八九十]?\s*日)?份?',
    r'\d{4}\s*\D\s*\d{1,2}\s*\D\s*\d{1,2}\s*\D{,2}\s*(?:\d{1,2}\s*\D\d{1,2}\s*\D\s*(?:\d{1,2})?)?',
])
P_NORMALIZE = (
    (P_DATES, ''),
    (regex.compile(r'(?<=\b[\u4e00-\u9fa5])\s*(?=[\u4e00-\u9fa5]\b)'), ''),
    (regex.compile(r'[∆|\s]+'), ' '),
)

P_INVALID_TEXT = hao.regexes.re_compile([
    r'[锟铰斤揭艿拷]{3,}',
])


def is_invalid_text(text: str) -> bool:
    if text is None:
        return True
    if hao.strings.is_invalid_chinese(text):
        return True
    if P_INVALID_TEXT.search(text) is not None:
        return True
    return False


def fix_text(text):
    if text is None:
        return ''
    text = fix_linebreaks(text)
    text = ftfy.fix_text(text)
    text = traditional_to_simplified(text)
    for p, sub in P_NORMALIZE:
        text = p.sub(sub, text)
    return text


def fix_linebreaks(text):
    text = regex.sub(r'([。！？\?])([^”’])', r"\1\n\2", text)  # 单字符断句符
    text = regex.sub(r'(\.{6})([^”’])', r"\1\n\2", text)  # 英文省略号
    text = regex.sub(r'(\…{2})([^”’])', r"\1\n\2", text)  # 中文省略号
    text = regex.sub(r'([。！？\?][”’])([^，。！？\?])', r'\1\n\2', text)
    return text.rstrip()


def traditional_to_simplified(text):
    converted = CC.convert(text)
    if len(set(converted) - set(text)) >= 10:
        return converted
    return text



# -*- coding: utf-8 -*-
import os

import fasttext
import hao
from hao.stopwatch import Stopwatch
from tailors.exceptions import TailorsError

from . import texts
from .tokenizer import JiebaTokenizer

LOGGER = hao.logs.get_logger(__name__)


class FastModel:

    def __init__(self,
                 path_or_key: str,
                 stopwords_path='data/dict/stopwords.txt',
                 keywords_path='data/dict/keywords.txt') -> None:
        super().__init__()
        self.path_or_key = path_or_key
        self.tokenizer = JiebaTokenizer(stopwords_path, keywords_path)
        self.model = self.load_model()

    def load_model(self):
        if ".local" in self.path_or_key:
            hao.oss.init(self.path_or_key[: self.path_or_key.rfind(".")])
            model_path = hao.config.get_path(self.path_or_key)
        else:
            model_path = hao.paths.get_path(self.path_or_key)

        if not os.path.exists(model_path):
            raise TailorsError(f"model path not exist: {model_path}")
        LOGGER.info(f"[FastModel] loading from: {model_path}")
        sw = Stopwatch()
        fasttext.FastText.eprint = lambda x: None
        model = fasttext.load_model(model_path)
        LOGGER.info(f"[FastModel] loaded, took: {sw.took()}")
        return model

    def predict(self, text: str):
        text = hao.strings.strip_to_none(text)
        if text is None:
            return None, None
        text = texts.fix_text(text)
        tokenized = self.tokenizer.cut_and_join(text)
        predictions, scores = self.model.predict(tokenized)
        prediction = predictions[0][9:]
        score = scores[0]
        return prediction, score



# -*- coding: utf-8 -*-
import hao

LOGGER = hao.logs.get_logger(__name__)

NUMBER_MAPPING = {
    '0': '零',
    '1': '一',
    '2': '二',
    '3': '三',
    '4': '四',
    '5': '五',
    '6': '六',
    '7': '七',
    '8': '八',
    '9': '九',
}


def generate_chinese_year_ranges(start=1950, end=2050):
    years = list()
    for year in range(start, end):
        year = str(year)
        chinese_year = list()
        for ch in year:
            chinese_year.append(NUMBER_MAPPING.get(ch, ch))
        years.append(''.join(chinese_year) + '年')
    return years



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tailors', 'tailors.fast', 'tailors.metrics', 'tailors.utils']

package_data = \
{'': ['*']}

install_requires = \
['einops>=0.4.1,<0.5.0',
 'fasttext>=0.9.2,<0.10.0',
 'ftfy>=6.1.1,<7.0.0',
 'hao>=3.7.0',
 'jieba>=0.42.1,<0.43.0',
 'opencc>=1,<2',
 'seqeval>=1.2.2,<2.0.0',
 'sklearn>=0.0,<0.1',
 'torch>=1.8.1',
 'transformers>=4.11,<5.0']

setup_kwargs = {
    'name': 'tailors',
    'version': '0.1.1',
    'description': '',
    'long_description': '# tailors\n\nabstract base models\n',
    'author': 'orctom',
    'author_email': 'orctom@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/orctom/tailors',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
import functools
from typing import List, Optional

import torch
from sklearn import metrics

precision_score = functools.partial(metrics.precision_score, average='micro', zero_division=0)
recall_score = functools.partial(metrics.recall_score, average='micro', zero_division=0)
f1_score = functools.partial(metrics.f1_score, average='micro', zero_division=0)
classification_report = functools.partial(metrics.classification_report, digits=4, zero_division=0)


def calculate(target, preds, labels: Optional[List[str]] = None):
    targets = target.tolist()
    return {
        'precision': precision_score(targets, preds, labels=labels),
        'recall': recall_score(targets, preds, labels=labels),
        'f1': f1_score(targets, preds, labels=labels),
        'report': classification_report(targets, preds, labels=labels)
    }



# -*- coding: utf-8 -*-
import os

import hao
import jieba
import regex
from tailors.utils import locations

from . import stopwords, texts

LOGGER = hao.logs.get_logger(__name__)

PUNCTUATIONS_EN = r'!"#$%&\'()*+,-./:;<=>?@[]^_`{|}~'
PUNCTUATIONS_ZH = r'＂＃＄￥％＆＇（）＊＋，－／：；＜＝＞＠［］＾＿｀｛｜｝～｟｠｢｣､\u3000、〃〈〉《》「」『』【】〔〕〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘’‛“”„‟…‧﹏﹑﹔·！？｡。'
PUNCTUATIONS_SPECIAL = '°∠г₂₂³×ɡΦΦφφФ′′ⅠⅣⅣ→≤≤≥≦°□こ'
PUNCTUATIONS = f'{PUNCTUATIONS_EN}{PUNCTUATIONS_ZH}{PUNCTUATIONS_SPECIAL}'


P_CITIES = hao.regexes.re_compile([c.strip('省市县镇村区') for c in locations.get_provinces_and_cities()])


P_NEG = hao.regexes.re_compile([
    rf'[{regex.escape(PUNCTUATIONS)}]',
    r'[一二三四五六七八九十百千万0-9]',
    r'[a-z]',
    r'^[A-Z]{,2}$',
    r'[一二三四五六七八九十百千万0-9]+[段届标期次批名个道封颗堵台首张根扇顶盘条幅件株位朵间头则片支峰篇只棵块粒匹座面方栋把枚斤批车平份人]',
    r'[ⅰⅱⅲⅳⅴⅵⅶⅷⅸⅹⅠⅡⅢⅣⅤⅥⅦⅧⅨХⅩⅪⅫ①②③④⑤⑥⑦⑧⑨⑩⒈⒉⒊⒋⒌⒍⒎⒏⒐⒑⑴⑵⑶⑷⑸⑹⑺⑻⑼⑽㈠㈡㈢㈣㈤㈥㈦㈧㈨㈩ⓐⓑⓒⓓⓔⓕⓖ]',
    r'[ΑαΒβΓγΔδΕεΖζΗηΘθΙιΚκΛλΜμΝνΞξΟοΠπΡρΣσΤτΥυΦφΧχΨψΩω]',
    r'[①-⑯⓵-⓾⒈-⒛➀-➉⑪-⑳⓿❶-❿➊-➓⓫-⓴⑴-⒇]',
    r'[_ᅳ]',
    r'[À-ÿ]',
    r'[〇ㄉㄊ㘵㳇至]',
    r'[段路村镇岭区街道寺线寨河海湖渡行庄府山桥港铺水]$',
    r'政采',
])


class AbstractTokenizer(object):

    def __init__(self) -> None:
        super().__init__()

    def tokenize(self, text):
        text = self.pre_tokenize(text)
        return self.cut(text)

    def pre_tokenize(self, text):
        return texts.fix_text(text)

    def cut(self, text):
        raise NotImplementedError()


class JiebaTokenizer(AbstractTokenizer):

    def __init__(self,
                 min_size: int = 1,
                 stopwords_path = 'data/dict/stopwords.txt',
                 keywords_path = 'data/dict/keywords.txt') -> None:
        super().__init__()
        self._min_size = min_size
        self._stopwords = stopwords.get_stopwords(stopwords_path)
        self._tokenizer = self._init_tokenizer(keywords_path)

    def _init_tokenizer(self, keywords_path):
        hao.paths.set_temp_dir('~/.tmp')
        tokenizer = jieba.Tokenizer()

        file_path = hao.path.get(keywords_path)
        if os.path.exists(file_path):
            with open(file_path) as f:
                for line in f:
                    line = hao.strings.strip_to_none(line)
                    if line is not None:
                        tokenizer.add_word(line)
        else:
            LOGGER.warning(f"keywords file not found: {keywords_path}")
        tokenizer.initialize()
        return tokenizer

    def cut(self, text):
        items = []
        for word in self._tokenizer.lcut(text):
            if len(items) > 0 and word == ' ' == items[-1]:
                continue
            if not self.is_valid_word(word):
                continue
            items.append(word.replace("\r", "").replace("\n", ""))
        return items

    def cut_and_join(self, text, sep=' '):
        return sep.join(self.cut(text))

    def is_valid_word(self, word):
        len_word = len(word)
        if self._min_size > 1 and len_word < self._min_size:
            return False
        if len_word == 1 and not hao.strings.is_char_chinese(word):
            return False
        if P_NEG.search(word) is not None:
            return False
        if word in self._stopwords:
            return False
        if P_CITIES.search(word) is not None:
            return False
        return True


class CharTokenizer(AbstractTokenizer):

    def cut(self, text):
        return list(text)
